/*
 * =======================================================================
 * FILE:        PFIFO.h
 * DESCRIPTION: Pointer FIFO
 * CREATED      20 MAR 10
 * MODIFIED     
 * PROGRAMMER:  William L. Bahn
 * COPYRIGHT:   2010
 *
 * These functions may be used for any purpose without royalty provided
 * the original author is acknowledged.
 * =======================================================================
 */

/*
 * =======================================================================
 * REVISION HISTORY
 *
 * REV 1.0: 20 MAR 10 Initial Creation.
 * =======================================================================
 */

//===========================================================================

//---------------------------------------------------------------------------
#ifndef PFIFO_DOT_H
#define PFIFO_DOT_H

#include "module.h"
#include "bytes.h"

//---------------------------------------------------------------------------
// Structure Type Definitions
//---------------------------------------------------------------------------
typedef struct PFIFO PFIFO;

//---------------------------------------------------------------------------
// Public Functions
//---------------------------------------------------------------------------

#define PFIFO_VERSION (0)
#define PFIFO_MAJOR   (1)
#define PFIFO_MINOR   (2)

WORD   PFIFO_version(int type);
void   PFIFO_report(PFIFO* p);
PFIFO* PFIFO_delete(PFIFO *p);
PFIFO* PFIFO_new(PFIFO *p, DWORD depth, DWORD width);

PFIFO *PFIFO_flush(PFIFO *p);
int    PFIFO_is_empty(PFIFO *p);
int    PFIFO_is_full(PFIFO *p);
int    PFIFO_count(PFIFO *p);
int    PFIFO_size(PFIFO *p);
int    PFIFO_width(PFIFO *p);

// NOTE: Pushing to a full FIFO will overwrite oldest data
void  *PFIFO_push(PFIFO *p, void *item);
// NOTE: Popping from an empty FIFO will return last data again
void  *PFIFO_pop(PFIFO *p, void *item);

int PFIFO_test(int type);

#endif
//---------------------------------------------------------------------------
