/*
 * =======================================================================
 * FILE:        PORT.c
 * DESCRIPTION: Software Defined Radio Block Port structure
 * CREATED      24 MAR 10
 * MODIFIED     
 * PROGRAMMER:  William L. Bahn
 * COPYRIGHT:   2010
 *
 * These functions may be used for any purpose without royalty provided
 * the original author is acknowledged.
 * =======================================================================
 */

/*
 * =======================================================================
 * REVISION HISTORY
 *
 * REV 1.0: 24 MAR 10 Initial Creation.
 * =======================================================================
 */

//===========================================================================

//---------------------------------------------------------------------------
// Include Files
//---------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "PORT.h"

//---------------------------------------------------------------------------
// Structure Definition
//---------------------------------------------------------------------------

#define MAJOR_REV (1)
#define MINOR_REV (0)

#define STRUCT PORT
#define CODESTRING "PORT"

struct STRUCT
{
	// Housekeeping Elements
	DWORD code;
	char *structname;
	// Live Elements
	int dir;      // PORT_OUTPUT
	PFIFO *fifo;  // NULL 
	int qty;      // 1
	int width;    // 32
	SDRB *from;   // NULL
};

#define CODE ((((((\
	(CODESTRING[0])<<8)\
	+CODESTRING[1])<<8)\
	+CODESTRING[2])<<8)\
	+CODESTRING[3])

//===========================================================================
// Primitive Functions
//===========================================================================
//---------------------------------------------------------------------------
// Housekeeping
//---------------------------------------------------------------------------

DWORD f(get_code)(STRUCT *p)
{
	return (p)? p->code : 0;
}

DWORD f(set_code)(STRUCT *p)
{
	if (p)
		p->code = CODE;

	return f(get_code)(p);
}

int f(structcheck)(STRUCT *p)
{
	if ((p)&&(f(get_code)(p)==CODE))
		return 1;
	return 0;
}

char *f(get_structname)(STRUCT *p)
{
	return (f(structcheck)(p))? p->structname : NULL;
}

char *f(set_structname)(STRUCT *p)
{
	if (f(structcheck)(p))
		p->structname = VNQUOTES(STRUCT);
	return f(get_structname)(p);
}

//---------------------------------------------------------------------------
// Element Creation/Deletion
//---------------------------------------------------------------------------

STRUCT *f(create)(void)
{
	STRUCT *p;

	p = (STRUCT *) malloc(sizeof(STRUCT));
	f(set_code)(p);
	f(set_structname)(p);

	return p;
}

STRUCT *f(destroy)(STRUCT *p)
{
	if (f(structcheck)(p))
	{
		free(p);
		p = NULL;
	}

	return p;
}

//---------------------------------------------------------------------------
// Element Access
//---------------------------------------------------------------------------

getset(int,    dir,   PORT_OUTPUT);
getset(PFIFO*, fifo,  NULL);
getset(int,    qty,    1);
getset(int,    width, 32);
getset(SDRB*,  from,  NULL);

//===========================================================================
// Private Functions
//===========================================================================

// The private functions perform little to no parameter checking.
// The public functions calling them are responsible for this.

void f(report_elements)(STRUCT* p, FILE *fp)
{
	fprintf(fp, "   dir      = %i\n", f(get_dir)(p));
	fprintf(fp, "   fifo     = %p\n", f(get_fifo)(p));
	fprintf(fp, "   qty      = %i\n", f(get_qty)(p));
	fprintf(fp, "   width    = %i\n", f(get_width)(p));
	fprintf(fp, "   from     = %p\n", f(get_from)(p));
}

//===========================================================================
// Public Functions
//===========================================================================

WORD f(version)(int type)
{
	switch (type)
	{
		case STRUCT_VERSION: return (MAJOR_REV<<8)+MINOR_REV; break;
		case STRUCT_MAJOR:   return MAJOR_REV; break;
		case STRUCT_MINOR:   return MINOR_REV; break;
	}
	return 0;
}

void f(report)(STRUCT* p, FILE *fp)
{
	if (NULL == fp)
		fp = stdout;

	fprintf(fp, "\n");
	fprintf(fp, "================================================\n");
	fprintf(fp, "Structure Name: %s (Code: %lX)\n", 
		f(get_structname)(p), f(get_code)(p));
	fprintf(fp, "Version: (%u.%u)\n", 
		f(version)(STRUCT_MAJOR), f(version)(STRUCT_MINOR));
	fprintf(fp, "------------------------------------------------\n");
	fprintf(fp, "   Size: %i bytes\n", sizeof(STRUCT));
	fprintf(fp, "   Addr: %p\n", p);
	fprintf(fp, "------------------------------------------------\n");
	f(report_elements)(p, fp);
	fprintf(fp, "================================================\n");
}

//---------------------------------------------------------------------------
// Constructor/Destructor
//---------------------------------------------------------------------------

int f(delete)(STRUCT *p)
{
	if (!p) return 1;

	free(p);

	return 0;
}

STRUCT *f(new)(void)
{
	STRUCT *p;
	
	p = NULL;

	p = f(create)();
	if (!p)
		return NULL;

	f(set_dir)   (p, f(get_dir)(NULL));
	f(set_fifo)  (p, f(get_fifo)(NULL));
	f(set_qty)   (p, f(get_qty)(NULL));
	f(set_width) (p, f(get_width)(NULL));
	f(set_from)  (p, f(get_from)(NULL));

	return p;
}

//---------------------------------------------------------------------------
// Primary Functions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Test Functions
//---------------------------------------------------------------------------

