/*
 * =======================================================================
 * FILE:        PORT.h
 * DESCRIPTION: Software Defined Radio Block Port structure
 * CREATED      24 MAR 10
 * MODIFIED     
 * PROGRAMMER:  William L. Bahn
 * COPYRIGHT:   2010
 *
 * These functions may be used for any purpose without royalty provided
 * the original author is acknowledged.
 * =======================================================================
 */

/*
 * =======================================================================
 * REVISION HISTORY
 *
 * REV 1.0: 24 MAR 10 Initial Creation.
 * =======================================================================
 */

//===========================================================================

//---------------------------------------------------------------------------
#ifndef PORT_DOT_H
#define PORT_DOT_H

#include <stdio.h>
#include "module.h"
#include "bytes.h"
#include "SDR.h"

//---------------------------------------------------------------------------
// Structure Type Definitions
//---------------------------------------------------------------------------
#define STRUCT PORT
typedef struct STRUCT STRUCT;

#define PORT_INPUT  (0)
#define PORT_OUTPUT (1)

//---------------------------------------------------------------------------
// Primitive Element Access Functions being made Public
//---------------------------------------------------------------------------

getset_proto(int,    dir,   PORT_OUTPUT);
getset_proto(PFIFO*, fifo,  NULL);
getset_proto(int,    qty,    1);
getset_proto(int,    width, 32);
getset_proto(SDRB*,  from,  NULL);

//---------------------------------------------------------------------------
// Public Functions
//---------------------------------------------------------------------------

WORD    f(version)(int type);
void    f(report)(STRUCT* p, FILE *fp);
int     f(delete)(STRUCT *p);
STRUCT* f(new)(void);

SDRB*   f(add_block)(STRUCT *p, char *name, 
			 SDRB_ModFunc ini, SDRB_ModFunc run, SDRB_ModFunc del, void *data);
SDRB*   f(add_monitor)(STRUCT *p, char *blockname);
PFIFO*  f(connect)(STRUCT *p, char *from, char *to, int fifo_depth);
int f(run)(STRUCT *p);

#undef STRUCT
#endif
//---------------------------------------------------------------------------
