/*
 * =======================================================================
 * FILE:        SDR.h
 * DESCRIPTION: Software Defined Radio structure
 * CREATED      24 MAR 10
 * MODIFIED     
 * PROGRAMMER:  William L. Bahn
 * COPYRIGHT:   2010
 *
 * These functions may be used for any purpose without royalty provided
 * the original author is acknowledged.
 * =======================================================================
 */

/*
 * =======================================================================
 * REVISION HISTORY
 *
 * REV 1.0: 24 MAR 10 Initial Creation.
 * =======================================================================
 */

//===========================================================================

//---------------------------------------------------------------------------
#ifndef SDR_DOT_H
#define SDR_DOT_H

//---------------------------------------------------------------------------
// Type Definitions required because of circular reference
//---------------------------------------------------------------------------

typedef struct PORT PORT;
typedef struct SDRB SDRB;
typedef struct PFIFO PFIFO;
typedef struct SDR SDR;
typedef int(*SDRB_ModFunc)(SDRB *, int when);
typedef int(*SDR_ModFunc)(SDR *, int when);

//---------------------------------------------------------------------------
// File Inclusions
//---------------------------------------------------------------------------

#include <stdio.h>
#include "module.h"
#include "bytes.h"
#include "PFIFO.h"
#include "PORT.h"
#include "SDRB.h"

//---------------------------------------------------------------------------
// Structure Type Definitions
//---------------------------------------------------------------------------
#define STRUCT SDR
typedef struct STRUCT STRUCT;

//---------------------------------------------------------------------------
// Primitive Element Access Functions being made Public
//---------------------------------------------------------------------------

getset_proto(int,     blocks,   0);
getset_proto(int,     fifos,    0);
getset_proto(int,     monitors, 0);
getset_proto(int,     errors,   0);
getset_proto(SDRB**,  block,    NULL);
getset_proto(PFIFO**, fifo,     NULL);
getset_proto(SDRB**,  monitor,  NULL);
getset_proto(void*,   data,     NULL);

//---------------------------------------------------------------------------
// Public Functions
//---------------------------------------------------------------------------

WORD    f(version)(int type);
void    f(report)(STRUCT* p, FILE *fp);
int     f(delete)(STRUCT *p);
STRUCT* f(new)(void *data, f(ModFunc) call);

int     f(errors)(STRUCT *p);
SDRB*   f(add_block)(STRUCT *p, char *name, SDRB_ModFunc call);
SDRB*   f(add_monitor)(STRUCT *p, char *blockname);
PFIFO*  f(connect)(STRUCT *p, char *from, char *to, int width, int depth);
int     f(reset)(STRUCT *p);
int     f(run)(STRUCT *p);
void   *f(datablock)(STRUCT *p);

//---------------------------------------------------------------------------
// Template Radio Block Function
//---------------------------------------------------------------------------
/*
int template(SDRB *p, int when)
{
	int retvalue;
	void *c;

	retvalue = 0; // Set retvalue to 1 to indicate errors.
	switch (when)
	{
		case SDRB_INI: // Initialation Behavior
			break;

		case SDRB_RST: // Reset Behavior
			break;

		case SDRB_RUN: // Runtime Behavior
			if ((!SDRB_is_empty(p, 0))&&(!SDRB_is_full(p, 0)))
			{
				SDRB_pop(p, 0, c);
				// Do something;
				SDRB_push(p, 0, c);
			}
			break;

		case SDRB_DEL: // Deletion Behavior
			break;

		default: // Should never reach this point.
			retvalue = 1;
	}
	return retvalue;
}
*/
//---------------------------------------------------------------------------

#undef STRUCT
#endif
//---------------------------------------------------------------------------
