/*
 * =======================================================================
 * FILE:        SDRB.h
 * DESCRIPTION: Software Defined Radio Block structure
 * CREATED      21 MAR 10
 * MODIFIED     
 * PROGRAMMER:  William L. Bahn
 * COPYRIGHT:   2010
 *
 * These functions may be used for any purpose without royalty provided
 * the original author is acknowledged.
 * =======================================================================
 */

/*
 * =======================================================================
 * REVISION HISTORY
 *
 * REV 1.0: 21 MAR 10 Initial Creation.
 * =======================================================================
 */

//===========================================================================

//---------------------------------------------------------------------------
#ifndef SDRB_DOT_H
#define SDRB_DOT_H

#include <stdio.h>
#include "module.h"
#include "bytes.h"
#include "SDR.h"

//---------------------------------------------------------------------------
// Structure Type Definitions
//---------------------------------------------------------------------------
#define STRUCT SDRB
typedef struct STRUCT STRUCT;
typedef int(*f(ModFunc))(STRUCT *, int);

#define SDRB_PORTS (16)
#define SDRB_INI (0)
#define SDRB_RST (1)
#define SDRB_RUN (2)
#define SDRB_DEL (3)

//---------------------------------------------------------------------------
// Primitive Element Access Functions being made Public
//---------------------------------------------------------------------------

getset_proto(char*,      name,     NULL); 
getset_proto(STRUCT*,    self,     NULL);
getset_proto(SDR*,       radio,     NULL);
getset_proto(int,        source,   FALSE);
getset_proto(int,        active,   FALSE);
getset_proto(void*,      data,     NULL);
getset_proto(f(ModFunc), call,      NULL);

getset_proto(PORT**,     in_port,   NULL);
getset_proto(PORT**,     out_port,  NULL);


//---------------------------------------------------------------------------
// Public Functions
//---------------------------------------------------------------------------

WORD  f(version)(int type);
void  f(report)(SDRB* p, FILE *fp);
int   f(delete)(SDRB *p);
STRUCT *f(new)(char *blockname, void *data, f(ModFunc) call);
int f(is_name)(STRUCT *p, char *name);
int f(is_full)(STRUCT *p, int port); // Is an output port full
int f(is_empty)(STRUCT *p, int port); // Is an input port empty
void *f(push)(STRUCT *p, int port, void *item);
void *f(pop)(STRUCT *p, int port, void *item);
//PFIFO *f(connect)(STRUCT *from, int from_port, STRUCT *to, int to_port, int depth);
//int f(is_port_active)(STRUCT *p, int port); // Is an input port still alive
PORT *f(new_port)(STRUCT *p, int dir, int num, int width, int qty, PFIFO *fifo, SDRB *from);

int f(is_active)(STRUCT *p);
int f(is_port_active)(STRUCT *p, int port);
int f(is_source)(STRUCT *p);
int f(mark_status)(STRUCT *p);
int f(call)(STRUCT *p, int when);
SDR *f(radio)(STRUCT *p);

#undef STRUCT
#endif
//---------------------------------------------------------------------------
