/*
 * =======================================================================
 * FILE:        STRUCT.c
 * DESCRIPTION: Structure Module Template File
 * CREATED      24 MAR 10
 * MODIFIED     
 * PROGRAMMER:  William L. Bahn
 * COPYRIGHT:   2010
 *
 * These functions may be used for any purpose without royalty provided
 * the original author is acknowledged.
 * =======================================================================
 */

/*
 * =======================================================================
 * REVISION HISTORY
 *
 * REV 1.0: 24 MAR 10 Initial Creation.
 * =======================================================================
 */

//===========================================================================

//---------------------------------------------------------------------------
// Include Files
//---------------------------------------------------------------------------

#include "STRUCT.h"

//---------------------------------------------------------------------------
// Structure Definition
//---------------------------------------------------------------------------

#define MAJOR_REV (1)
#define MINOR_REV (0)

#define STRUCT FRED
#define CODESTRING "FRED"

struct STRUCT
{
	// Housekeeping Elements
	DWORD code;
	char *structname;
	// Live Elements
	char *name;      // NULL
	PFIFO *fifo_in;  // NULL 
	PFIFO *fifo_out; // NULL
	int q_in;        // 0
	int q_out;       // 0
	int w_in;        // 0
	int w_out;       // 0
	int running;     // FALSE
	SDRB *self;      // NULL
	SDRB *kill;      // NULL
	void *data;      // NULL
	f(ModFunc) ini;  // NULL
	f(ModFunc) run;  // NULL
	f(ModFunc) del;  // NULL
};

#define CODE ((((((\
	(CODESTRING[0])<<8)\
	+CODESTRING[1])<<8)\
	+CODESTRING[2])<<8)\
	+CODESTRING[3])

//===========================================================================
// Primitive Functions
//===========================================================================
//---------------------------------------------------------------------------
// Housekeeping
//---------------------------------------------------------------------------

DWORD f(get_code)(STRUCT *p)
{
	return (p)? p->code : 0;
}

DWORD f(set_code)(STRUCT *p)
{
	if (p)
		p->code = CODE;

	return f(get_code)(p);
}

int f(structcheck)(STRUCT *p)
{
	if ((p)&&(f(get_code)(p)==CODE))
		return 1;
	return 0;
}

char *f(get_structname)(STRUCT *p)
{
	return (f(structcheck)(p))? p->structname : NULL;
}

char *f(set_structname)(STRUCT *p)
{
	if (f(structcheck)(p))
		p->structname = VNQUOTES(STRUCT);
	return f(get_structname)(p);
}

STRUCT *f(create)(void)
{
	STRUCT *p;

	p = (STRUCT *) malloc(sizeof(STRUCT));
	f(set_code)(p);
	f(set_structname)(p);

	return p;
}

STRUCT *f(destroy)(STRUCT *p)
{
	if (f(structcheck)(p))
	{
		free(p);
		p = NULL;
	}

	return p;
}

//---------------------------------------------------------------------------
// Element Access
//---------------------------------------------------------------------------

getset(char*,      name,     NULL); 
getset(PFIFO*,     fifo_in,  NULL); 
getset(PFIFO*,     fifo_out, NULL);
getset(int,        q_in,     0);
getset(int,        q_out,    0);
getset(int,        w_in,     0);
getset(int,        w_out,    0);
getset(int,        running,  FALSE);
getset(SDRB*,      self,     NULL);
getset(SDRB*,      kill,     NULL);
getset(void*,      data,     NULL);
getset(f(ModFunc), ini,       NULL);
getset(f(ModFunc), run,       NULL);
getset(f(ModFunc), del,       NULL);

//===========================================================================
// Private Functions
//===========================================================================

// The private functions perform little to no parameter checking.
// The public functions calling them are responsible for this.

void f(report_elements)(STRUCT* p)
{
	printf("   name     = %s\n", f(get_name)(p));
	printf("   fifo_in  = %p\n", f(get_fifo_in)(p));
	printf("   fifo_out = %p\n", f(get_fifo_out)(p));
	printf("   q_in     = %i\n", f(get_q_in)(p));
	printf("   q_out    = %i\n", f(get_q_out)(p));
	printf("   w_in     = %i\n", f(get_w_in)(p));
	printf("   w_out    = %i\n", f(get_w_out)(p));
	printf("   running  = %i\n", f(get_running)(p));
	printf("   self     = %p\n", f(get_self)(p));
	printf("   kill     = %p\n", f(get_kill)(p));
	printf("   data     = %i\n", f(get_data)(p));
	printf("   ini      = %p\n", f(get_ini)(p));
	printf("   run      = %p\n", f(get_run)(p));
	printf("   del      = %p\n", f(get_del)(p));
}

//===========================================================================
// Public Functions
//===========================================================================

WORD f(version)(int type)
{
	switch (type)
	{
		case STRUCT_VERSION: return (MAJOR_REV<<8)+MINOR_REV; break;
		case STRUCT_MAJOR:   return MAJOR_REV; break;
		case STRUCT_MINOR:   return MINOR_REV; break;
	}
	return 0;
}

void f(report)(STRUCT* p)
{
	printf("\n");
	printf("================================================\n");
	printf("Structure Name: %s (Code: %lX)\n", 
		f(get_structname)(p), f(get_code)(p));
	printf("Version: (%u.%u)\n", 
		f(version)(STRUCT_MAJOR), f(version)(STRUCT_MINOR));
	printf("------------------------------------------------\n");
	printf("   Size: %i bytes\n", sizeof(STRUCT));
	printf("   Addr: %p\n", p);
	printf("------------------------------------------------\n");
	f(report_elements)(p);
	printf("================================================\n");
}

//---------------------------------------------------------------------------

int f(delete)(STRUCT *p)
{
	if (!p) return 1;

	// Attached FIFOs are NOT deleted by this function.
	// They must be destroyed independently

	// Invoke Block-specific destructor
	if (f(get_del)(p))
		(*(f(get_del)(p)))(f(get_self)(p));

	free(p);
	return 0;
};

STRUCT *f(new)(char *name, f(ModFunc) ini, f(ModFunc) run, f(ModFunc) del, void *data)
{
	STRUCT *p;
	
	p = NULL;

	if (!name)
		return NULL;

	p = f(create)();
	if (!p)
		return NULL;

	f(set_name)     (p, f(get_name)(NULL));
	f(set_self)     (p, f(get_self)(NULL));
	f(set_kill)     (p, f(get_kill)(NULL));
	f(set_fifo_in)  (p, f(get_fifo_in)(NULL));
	f(set_fifo_out) (p, f(get_fifo_out)(NULL));
	f(set_q_in)     (p, f(get_q_in)(NULL));
	f(set_q_out)    (p, f(get_q_out)(NULL));
	f(set_w_in)     (p, f(get_w_in)(NULL));
	f(set_w_out)    (p, f(get_w_out)(NULL));
	f(set_running)  (p, f(get_running)(NULL));
	f(set_data)     (p, f(get_data)(NULL));
	f(set_ini)      (p, f(get_ini)(NULL));
	f(set_run)      (p, f(get_run)(NULL));
	f(set_del)      (p, f(get_del)(NULL));

	f(set_name)     (p, name);
	f(set_self)     (p, p);
	f(set_ini)      (p, ini);
	f(set_run)      (p, run);
	f(set_del)      (p, del);
	f(set_data)     (p, data);
	f(set_running)  (p, TRUE);

	if (f(get_ini)(p))
		(*(f(get_ini)(p)))(f(get_self)(p));

	return p;
}

int f(is_name)(STRUCT *p, char *name)
{
	return (!strcmp(f(get_name)(p), name));
}

PFIFO *f(connect)(STRUCT *from, int from_port, STRUCT *to, int to_port, int depth)
{
	PFIFO *fifo;

	fifo = NULL;

	// NOTE: Port numbers not implemented yet.

	// At least one of the blocks must be defined.
	if ((!from)&&(!to))
		return NULL;

	// Special case : Signal Source (from == NULL)
	if (!from)
	{
		f(set_fifo_in)(to, NULL);
		f(set_kill)(to, NULL);
	}

	// Special case : Signal Sink (to == NULL)
	if (!to)
	{
		f(set_fifo_out)(from, NULL);
	}

	// Normal case : If both blocks exist must have equal port widths.
	if((from)&&(to))
	{
		if (f(get_w_out)(from) == f(get_w_in)(to))
		{
			fifo = PFIFO_new(NULL, depth, from->w_out);
			f(set_fifo_out)(from, fifo);
			f(set_fifo_in)(to, fifo);
			f(set_kill)(to, from);
			if ((!f(get_fifo_out)(from))||(!f(get_fifo_in)(to)))
				return NULL;
		}
		else
		{
			return NULL;
		}
	}

	return fifo;
}

int f(run)(STRUCT *p)
{
	if (!p) return 1;

	//if (p->q_in  > PFIFO_count(p->fifo_in)) return 0;
	//if (p->q_out > (PFIFO_size(p->fifo_out) - PFIFO_count(p->fifo_out))) return 0;
	if (f(get_run)(p))
		(*(f(get_run)(p)))(p);

	return 0;
}

